@@ -5,14 +5,17 @@
 #include <linux/xattr.h>
 
 #include "posix_to_bcachefs.h"
-#include "libbcachefs/alloc_foreground.h"
-#include "libbcachefs/buckets.h"
-#include "libbcachefs/io_misc.h"
-#include "libbcachefs/io_read.h"
-#include "libbcachefs/io_write.h"
-#include "libbcachefs/namei.h"
-#include "libbcachefs/str_hash.h"
-#include "libbcachefs/xattr.h"
+
+#include "alloc/foreground.h"
+#include "alloc/buckets.h"
+
+#include "data/io_misc.h"
+#include "data/read.h"
+#include "data/write.h"
+
+#include "fs/namei.h"
+#include "fs/str_hash.h"
+#include "fs/xattr.h"
 
 struct hardlink {
 	struct rhash_head	hash;
@@ -280,7 +283,7 @@ static void write_data(struct bch_fs *c,
 	bch2_bio_map(&op.wbio.bio, buf, len);
 
 	struct bch_inode_opts opts;
-	bch2_inode_opts_get(c, &opts);
+	bch2_inode_opts_get(c, &opts, false);
 
 	bch2_write_op_init(&op, c, opts);
 	op.write_point	= writepoint_hashed(0);
@@ -351,7 +354,7 @@ static void link_data(struct bch_fs *c, struct bch_inode_unpacked *dst,
 		e->k.p.offset	= logical + sectors;
 		e->k.p.snapshot	= U32_MAX;
 		e->k.size	= sectors;
-		bch2_bkey_append_ptr(&e->k_i, (struct bch_extent_ptr) {
+		bch2_bkey_append_ptr(c, &e->k_i, (struct bch_extent_ptr) {
 					.offset = physical,
 					.dev = 0,
 					.gen = *bucket_gen(ca, b),
